package de.hdm_stuttgart.mi.sd1.test.task1;

import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.task1.Helper;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_Helper extends ExaminationTestDefaults {

  /* Section: isAscending */

  @Test @Marking(points = 3) public void test_010_isAscending_positive() {
    Assert.assertTrue(Helper.isAscending(1, 2, 3));
    Assert.assertFalse("Second value not greater than first", Helper.isAscending(1, 1, 3));
    Assert.assertFalse("All values are strictly descending", Helper.isAscending(3, 2, 1));
  }

  @Test @Marking(points = 3) public void test_020_isAscending_negative() {
    Assert.assertTrue(Helper.isAscending(-41, -20, 30));
    Assert.assertFalse("Second value not greater than first", Helper.isAscending(-50, -50, 3));
  }


  /* Section: checkTitle */

  @Test @Marking(points = 2) public void test_210_checkTitle_null() {
    Assert.assertFalse("a null title is invalid", Helper.checkTitle(null));
    Assert.assertTrue(Helper.checkTitle("This is definitely fine"));
  }
  @Test @Marking(points = 2) public void test_checkTitle_010_tooShort() {
    Assert.assertFalse("title too short", Helper.checkTitle("A"));
    Assert.assertFalse("title too short", Helper.checkTitle("Abcde"));
    Assert.assertFalse("title too short", Helper.checkTitle("A123456789012345678"));
    Assert.assertTrue( Helper.checkTitle("A1234567890123456789"));
    Assert.assertTrue( Helper.checkTitle("A12345678901234567890"));
  }

  @Test @Marking(points = 1) public void test_220_checkTitle_smallCaps() {
    Assert.assertFalse("title too short and first char small caps", Helper.checkTitle("a"));
    Assert.assertFalse("title too short and first char small caps", Helper.checkTitle("abcde"));
    Assert.assertFalse("title too short and first char small caps", Helper.checkTitle("a123456789012345678"));
    Assert.assertFalse("first char small caps", Helper.checkTitle("a1234567890123456789"));
    Assert.assertFalse("first char small caps", Helper.checkTitle("a12345678901234567890"));
    Assert.assertTrue( Helper.checkTitle("A12345678901234567890"));
  }

  @Test @Marking(points = 1) public void test_225_checkTitle_letter() {

      Assert.assertFalse("Invalid: Non-letter (digit) at beginning",
              Helper.checkTitle("4all of you: This is bullshit!"));
      Assert.assertFalse("Invalid: Non-letter(underscore) at beginning",
              Helper.checkTitle("_all of you: This is bullshit!"));
    Assert.assertTrue(Helper.checkTitle("For all of you: This is valid!"));
    Assert.assertTrue(Helper.checkTitle("A12345678901234567890"));
  }

  @Test @Marking(points = 2) public void test_230_checkTitle_Small() {
    Assert.assertFalse("title too short and first char small caps", Helper.checkTitle("too short"));
    Assert.assertFalse("title too short", Helper.checkTitle("Too short"));
    Assert.assertTrue( Helper.checkTitle("A12345678901234567890"));
  }

  @Test @Marking(points = 2) public void test_240_checkTitle_Long() {
    Assert.assertFalse("first char small caps", Helper.checkTitle("you should not start with small caps"));
    Assert.assertTrue(Helper.checkTitle("You should not start with small caps"));
  }


  /* Section: isVowel(...) */

  @Test @Marking(points = 5) public void test_410_checkVowel() {

    for (final char consonant: new char[]{
            'b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's', 't', 'v', 'w', 'x', 'y', 'z'}) {
      Assert.assertFalse(Helper.isVowel(consonant));
      Assert.assertFalse(Helper.isVowel(Character.toUpperCase(consonant)));
    }
    for (final char vowel: new char[]{'a', 'e', 'i', 'o', 'u'}) {
      Assert.assertTrue(Helper.isVowel(vowel));
      Assert.assertTrue(Helper.isVowel(Character.toUpperCase(vowel)));
    }
  }

  /* Section: getNumberOfVowels(...) */

  @Test @Marking(points = 2) public void test_420_getNumberOfVowels_null() {
    Assert.assertEquals(0, Helper.getNumberOfVowels(null));
  }

  @Test @Marking(points = 2) public void test_430_getNumberOfVowels() {
    Assert.assertEquals(0, Helper.getNumberOfVowels(""));
    Assert.assertEquals(0, Helper.getNumberOfVowels("Thx"));
    Assert.assertEquals(1, Helper.getNumberOfVowels("O"));
    Assert.assertEquals(3, Helper.getNumberOfVowels("Oui"));
    Assert.assertEquals(7, Helper.getNumberOfVowels("This is quite simple."));
  }

  /* Section: renameFileBasename */

  @Test @Marking(points = 2) public void test_610_rename_noExtension() {
    Assert.assertEquals("standard", Helper.renameFileBasename("standard", "standard"));
    Assert.assertEquals("other", Helper.renameFileBasename("standard", "other"));
  }
  @Test @Marking(points = 3) public void test_620_rename_simpleExtension() {
      Assert.assertEquals("doo.pdf", Helper.renameFileBasename("foo.pdf", "doo"));
      Assert.assertEquals("doo.pdf", Helper.renameFileBasename(".pdf", "doo"));
  }
  @Test @Marking(points = 3) public void test_630_rename_complexExtension() {
    Assert.assertEquals("new.tar.gz", Helper.renameFileBasename("archive.tar.gz", "new"));
    Assert.assertEquals("cute.b.c.d.e.f", Helper.renameFileBasename("a.b.c.d.e.f", "cute"));
    Assert.assertEquals("cute.b.c.d.e.f", Helper.renameFileBasename("a.b.c.d.e.f", "cute"));
  }
}