package de.hdm_stuttgart.mi.sd1.test.task1;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import java.util.Arrays;

import static de.hdm_stuttgart.mi.sd1.task1.HelperArray.getAlphabeticalFollowers;
import static de.hdm_stuttgart.mi.sd1.task1.HelperArray.getLeaders;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_HelperArray extends ExaminationTestDefaults {

  private static void assertArrayEmpty(final String[] values){
    Assert.assertArrayEquals(
        new String[]{},
        values);
  }

  /* Section: getAlphabeticalFollowers */

  @Test @Marking(points = 4) public void test_400_testSpecialCases() {

    assertArrayEmpty(getAlphabeticalFollowers(null, null));

    assertArrayEmpty(getAlphabeticalFollowers(null, ""));

    assertArrayEmpty(getAlphabeticalFollowers(null, "garbage"));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, null));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, ""));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, "garbage"));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{"Some Value"}, null));
  }

  private void assertFilter(final String[] expected, final String[] values, final String reference) {

    final String [] valuesCopy = Arrays.copyOf(values, values.length);

    final String[] result = getAlphabeticalFollowers(values, reference);

    Assert.assertNotSame("filtered array must not be equal to input", result, values);

    Assert.assertArrayEquals(expected, result);

    Assert.assertArrayEquals("Error: Original array value modifications", valuesCopy, values);

  }

  @Test @Marking(points = 3) public void test_460_testNormal() {

    assertFilter(new String[]{"pi", "sigma", "pi"},
                 new String[]{"alpha", "pi", "delta", "sigma", "pi"}, "gamma");

    assertFilter(new String[]{"pi", "delta", "sigma", "pi"},
              new String[]{"alpha", "pi", "delta", "sigma", "pi"}, "alpha");

    assertFilter(new String[]{"pi", "delta", "sigma", "pi"},
              new String[]{"pi", "delta", "alpha", "sigma", "pi"}, "alpha");

    assertFilter(new String[]{"pi", "delta", "sigma", "pi"},
              new String[]{"pi", "delta", "sigma", "pi", "alpha"}, "alpha");
  }

  @Test @Marking(points = 3) public void test_480_testNull() {
    assertFilter(new String[]{"sigma"}, new String[]{"alpha", null, "delta", "sigma", null}, "gamma");
  }

  @Test @Marking(points = 2)  public void test_500_testEmpty() {
    assertArrayEmpty(getAlphabeticalFollowers(new String[]{}, "alpha"));

    assertArrayEmpty(getAlphabeticalFollowers(new String[]{"alpha", "pi", "delta", "pi"}, "theta"));
    
    assertArrayEmpty(getAlphabeticalFollowers(new String[]{""}, ""));
  }

  @Test @Marking(points = 2) public void test_520_testAll() {

    final String[] values = {"alpha", "pi", "delta", "pi"};

    assertFilter(values, values, "");
    assertFilter(values, values, "alph");
  }


  /* Section: getLeaders */


  @Test public void test_600_singleLeader() {
    Assert.assertArrayEquals(new int[]{-3}, getLeaders(new int[]{-3}));
    Assert.assertArrayEquals(new int[]{1, 0}, getLeaders(new int[]{1, 0}));
    Assert.assertArrayEquals(new int[]{1}, getLeaders(new int[]{0, 1}));
  }

  @Test public void test_620_edgeDuplicate() {
    Assert.assertArrayEquals(new int[]{3, 3}, getLeaders(new int[]{1, 3, -2, 0, 3}));
    Assert.assertArrayEquals(new int[]{4, 3}, getLeaders(new int[]{1, 4, -2, 0, 3}));
    Assert.assertArrayEquals(new int[]{3}, getLeaders(new int[]{1, 2, -2, 0, 3}));
    Assert.assertArrayEquals(new int[]{3, 3}, getLeaders(new int[]{3, 2, -2, 0, 3}));
    Assert.assertArrayEquals(new int[]{6, 3}, getLeaders(new int[]{6, 2, -2, 0, 3}));
    Assert.assertArrayEquals(new int[]{17, 5, 2}, getLeaders(new int[]{16, 17, 4, 3, 5, 2}));
  }

  @Test public void test_640_all() {
    Assert.assertArrayEquals(new int[]{5,4,3,2,1,0}, getLeaders(new int[]{5,4,3,2,1,0}));
    Assert.assertArrayEquals(new int[]{4,4,4,4,4,4,4}, getLeaders(new int[]{4,4,4,4,4,4,4}));
  }
}